01000 ;Example NextZXOS driver
01010 ;by Simon N Godwin 5-2021
01020 ;to be built natively :-)
01030 ;
01040 ORG 0      ;Relocate base
01050 APAGE EQU 8192 ;Page size
01060 ;Bank 6 base address for
01070 ;code and line number data
01080 PAGE6 EQU 49152
01090 DISP PAGE6 ;Object address
01100 PPC   EQU 23621 ;Sys Var
01105 ERRNR EQU 23610 ;Sys Var
01110 BANKS EQU 3  ;20K needed
01120 LIMIT EQU 512 ;Driver size
01130 ENT
01140 ;Entry point for commands
01150 API   JR  COMMS
01160       NOP ;Offset 3 pad
01170 ;Entry point for interrupt
01180 INTR  JR PROFI
01190 ;
01200 COMMS LD  A,B ;DRIVER B,
01210       OR  A
01220       JR  NZ,NOT0
01230       OR  D ;DRIVER 0,0?
01240       OR  E
01250       JR  Z,RESET ;0,0
01260 ;DRIVER 0,Line Zero count
01270 ;Check that DE <= 9999
01280       LD  HL,9999 ;Cy=0
01290       SBC HL,DE
01300       JR  C,FAIL
01310 R0    CALL PAGER
01320 R1    CALL FINDL
01325       PUSH BC
01330       LD  C,(HL) ;PreRead
01340       XOR A ; Clear count
01350       LD (HL),A
01360       INC HL ;For HiByte
01370       LD  B,(HL)
01380       LD (HL),A
01383       LD  L,C
01385       LD  H,B
01387       POP BC
01390       JR  DONE ;OK, Cy=0
01400 ;
01410 ;DRIVER 1,Line TO Count
01420 NOT0  DJNZ NOT1
01422       LD  A,D
01424       OR  E
01427       JR  Z,FAIL ;Line 0!
01430       LD  HL,9999
01440       SBC HL,DE
01450       JR  C,FAIL ;DE>9999
01460 R2    CALL PAGER
01470 R3    CALL FINDL
01480       LD  A,(HL) ;Count
01490       INC HL ;For HiByte
01500       LD  H,(HL)
01505       LD  L,A
01510       JR  DONE ;OK, Cy=0
01520 NOT1  DJNZ NOT2
01530       OR  A     ;Clear Cy
01540 R4    LD  HL,LIMIT-1
01550       DEC DE ;Start at 1
01560       SBC HL,DE ;Cy=error
01570       LD  C,(HL)
01580       RET ;Success if Cy=0
01590 NOT2  DJNZ FAIL
01600 ;Could add command 3 here
01620 ;
01630 ;Unmatched B, flag error
01640 FAIL  XOR A
01650       SCF ;Cy=1, Error
01660       RET
01670 ;
01680 ;Clear all counts
01690 RESET CALL PAGER
01700 R5    LD  HL,LIMIT-1
01710       PUSH DE ;Old state
01720       PUSH BC ;DATA reg
01730       LD  B,BANKS
01740 BLOOP LD  A,(HL)
01743 ;Use NEXTREG 86,A opcode
01746 ;to map bank A to PAGE6
01750       DEFB 237,146,86
01760       XOR A
01770       PUSH HL
01780       PUSH BC
01790       LD DE,PAGE6+1 ;Dest
01800       LD HL,PAGE6 ;Source
01810       LD  (HL),A ;Clear 1
01820       LD  BC,APAGE-1
01830       LDIR   ; Clear page
01840       POP BC
01850       POP HL
01860       DEC HL
01870       DJNZ BLOOP
01880       POP BC  ;Port addr
01890       POP DE  ;Port state
01900       JR  DONE
01910 ;
01920 ;BASIC Profile handler
01930 PROFI LD  A,(ERRNR)
01934       INC A ;255=Running
01936       JR  NZ,LINE0 ;Stop
01938       LD  DE,(PPC) ;Line
01940       LD  A,D
01950       OR  E
01960       JR  Z,LINE0
01970 R6    CALL PAGER
01980 R7    CALL FINDL
02090       INC (HL) ;Low byte
02100       JR  NZ,DONE
02110 ;Carry to high byte
02120       INC HL
02130       INC (HL)
02140       JR  NC,DONE
02150 ;OVERFLOW! Clamp to 65535
02160       DEC (HL)
02170       DEC HL
02180       DEC (HL)
02190 ;Restore banks and return
02200 ;D=Old SELECT register
02210 ;E=Old slot 6 bank BC=DATA
02220 DONE  LD  A,E ;Reset bank
02230       DEFB 237,146,86
02240 ;Restore select
02250       DEC B
02260       OUT (C),D
02263       LD B,H
02265       LD C,L
02270 LINE0 RET
02280 ;
02300 ;Find slot for linenum HL
02310 FINDL LD  A,15 ;Line HL
02320       AND H
02325       PUSH DE
02330 ;Save address bits 8..11
02340       LD  D,A
02350 ;Get bank # 0..2 into A
02360       LD  A,#30 ;00110000
02370       AND H ;Bits 12 & 13
02380       RRCA ;Divide by 16
02390       RRCA
02400       RRCA ;to bits 0 & 1
02410       RRCA
02420 ;Restore HL (LNUM & 4095)
02430       LD  H,D ;HL 0..4095
02440       ADD HL,HL ; 0..8190
02450 ;Offset to bank base
02460       LD  DE,PAGE6 ;8K*6
02470       ADD HL,DE
02490 ;Bank numbers at 509..511
02500       EX  DE,HL
02510 R8    LD  HL,HIGHSTREET
02520       ADD A,L ;L=253..255
02530 ;Save current SELECT port
02540       LD  L,A ;HL->Bank #
02550       LD  A,(HL) ;A=Bank
02560       EX  DE,HL
02570       POP DE
02580 ;Use NEXTREG 86,A opcode
02590       DEFB 237,146,86
02600 ;Bank is now at 49152
02800       RET       ;HL->slot
02810 ;
02820 ;Find bank configuration
02830 PAGER EX  DE,HL ;So HL=DE
02840       LD  BC,9275 ;Select
02850       IN  D,(C)
02860       LD  E,86
02870       OUT (C),E ;Get MMU6
02880       INC B
02890       IN  E,(C) ;IN 9531
02900 COUNT RET
02910 ;
02920 ;Variables and constants
02930 ORG LIMIT-BANKS ; Bank #s
02940 HIGHSTREET DEFB 0,0,0
02950 ;
02960 ;High bytes of addresses
02970 RELOCS EQU $ ;512 (LIMIT)
02980       DEFW R0+2
02990       DEFW R1+2
03000       DEFW R2+2
03010       DEFW R3+2
03020       DEFW R4+2
03030       DEFW R5+2
03040       DEFW R6+2
03050       DEFW R7+2
03060       DEFW R8+2
03070       DEFW RESET+2
03080 LENGTH EQU $
